"""
This module contains the assertions and error definitions.


"""
from schul_cloud_resources_server_tests.tests.assertions import assertIsError

ERROR_CLIENT_REQUEST = 409 # https://httpstatuses.com/409
Q = "Q" # The query string
